package com.monkeycoder.monkeynes.base;

import java.io.*;
import java.util.*;

/**
 * This class is a logger singleton that can direct log output to several places.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class DebugLog
{
  public static final int MODE_NULL       = 0;    // Debug output is ignored
  public static final int MODE_STD_OUT    = 1;    // Debug is directed to System.out
  public static final int MODE_STD_ERR    = 2;    // Debug is directed to System.err
  public static final int MODE_FILE       = 3;    // Debug is appended to the file "debug.log"
  
  private static DebugLog   inst          = null;
  private        int        mode          = MODE_NULL;
  private        int        prevMode      = MODE_NULL;
  private        int        vThresh       = 0;  // only things with this verbosity or lower display (defaults to nothing showing up)
  private        int        prevVThresh   = 0;
  private        FileWriter file          = null;
  
  private DebugLog()
  {
    try
    {
      file = new FileWriter( "debug.log", true );
      file.write( "=================================================\r\n" );
      file.write( "Log Started: " + new Date() + "\r\n" );
      file.write( "=================================================\r\n" );
      file.flush();
    }
    catch( IOException ioe )
    {}
  }
  
  /**
   * Returns the only allowed instance of this class.
   * @return The DebogLog instance.
   */  
  public static DebugLog getInst()
  {
    if( inst == null )
      inst = new DebugLog();
    
    return inst;  
  }

  /**
   * Internal function that sets the current redirection mode.
   * @param inMode The redirection mode to which the DebugLog will be set.
   */  
  private void iSetMode( int inMode )
  {
    prevMode = mode;
    mode     = inMode;
  }
  
  /**
   * Internal function that returns the current redirection mode.
   * @return The redirection mode.
   */
  private int iGetMode()
  {
    return mode;
  }
  
  /**
   * Internal function that puts the DebugLog back into its previous redirection mode.
   */  
  private void iRestoreLastMode()
  {
    mode = prevMode;
  }  
  
  /**
   * Internal function that sets the current verbosity threshhold.
   * @param inVThresh The verbosity threshhold to which the DebugLog will be set.
   */  
  private void iSetVerbosity( int inVThresh )
  {
    prevVThresh = vThresh;
    vThresh     = inVThresh;
  }
  
  /**
   * Internal function that returns the current verbosity threshhold.
   * @return The verbosity threshhold.
   */  
  private int iGetVerbosity()
  {
    return vThresh;
  }
  
  /**
   * Internal function that puts the DebugLog back into its previous verbosity threshhold.
   */  
  private void iRestoreLastVerbosity()
  {
    vThresh = prevVThresh;
  }   
  
  /**
   * Internal function for recording a log entry.
   * @param inVerbosity The verbosity threshhold required to see this log entry.
   * @param inText The text for the log entry.
   */ 
  private void iLog( int inVerbosity, String inText )
  {
    try
    {
      if( inVerbosity <= vThresh )
      {
        switch( mode )
        {
          case MODE_NULL:
            break;
          case MODE_STD_OUT:
            System.out.println( inText );
            break;
          case MODE_STD_ERR:
            System.err.println( inText );
            break;  
          case MODE_FILE:
            file.write( inText + "\r\n" );
            file.flush();
            break;  
        }
      }  
    }
    catch( IOException ioe )
    {}
  }
  
  // The publicly accessible static functions --------------------------
  
  /**
   * Records a log entry using the maximum verbosity threshhold of 5.
   * @param inText The text for the log entry.
   */
  public static void log( String inText )
  {
    DebugLog.getInst().iLog( 5, inText );
  }

  /**
   * Records a log entry using the provided verbosity threshhold value.
   * @param inVerbosity The verbosity threshhold required to see this log entry.
   * @param inText The text for the log entry.
   */  
  public static void log( int inVerbosity, String inText )
  {
    DebugLog.getInst().iLog( inVerbosity, inText );
  }  
  
  /**
   * Sets the current redirection mode.
   * @param inMode The redirection mode to which the DebugLog will be set.
   */  
  public static void setMode( int inMode )
  {
    DebugLog.getInst().iSetMode( inMode );
  }

  /**
   * Returns the current redirection mode.
   * @return The redirection mode.
   */  
  public static int getMode()
  {
    return DebugLog.getInst().iGetMode();
  }

  /**
   * Puts the DebugLog back into its previous redirection mode.
   */   
  public static void restoreLastMode()
  {
    DebugLog.getInst().iRestoreLastMode();
  }  

  /**
   * Sets the current verbosity threshhold.
   * @param inVThresh The verbosity threshhold to which the DebugLog will be set.
   */  
  public static void setVerbosity( int inVThresh )
  {
    DebugLog.getInst().iSetVerbosity( inVThresh );
  }

  /**
   * Returns the current verbosity threshhold.
   * @return The verbosity threshhold.
   */   
  public static int getVerbosity()
  {
    return DebugLog.getInst().iGetVerbosity();
  }

  /**
   * Puts the DebugLog back into its previous verbosity threshhold.
   */   
  public static void restoreLastVerbosity()
  {
    DebugLog.getInst().iRestoreLastVerbosity();
  }  
  
}