package com.monkeycoder.monkeynes.hardware;

/**
 * This class represents an 8-bit CPU register.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class CpuRegister
{
  // Known status register bitmasks
  public static final int PS_CARRY_FLAG           = 0x00000001;    // C
  public static final int PS_ZERO_FLAG            = 0x00000002;    // Z
  public static final int PS_INTERRUPT_DISABLE    = 0x00000004;    // I
  public static final int PS_DECIMAL_MODE         = 0x00000008;    // D - Not supported by NES
  public static final int PS_BREAK_COMMAND        = 0x00000010;    // B
  public static final int PS_UNUSED               = 0x00000020;    // not used - supposed to be 1
  public static final int PS_OVERFLOW_FLAG        = 0x00000040;    // V
  public static final int PS_NEGATIVE_FLAG        = 0x00000080;    // S - S for negative?  That person should be shot.
  
  private int data = 0;

  /**
   * Creates a new CpuRegister set to 0.
   */
  public CpuRegister()
  {
    data = 0;
  }

  /**
   * Creates a new CpuRegister set to a specific value.
   * @param inData The value to which the register will be set.
   */
  public CpuRegister( int inData )
  {
    setValue( inData );
  }

  /**
   * Gets the entire data value of this register.
   * @return The value of this register as an int.
   */
  public int getValue()
  {
    return data;
  }

  /**
   * Gets the value of a specific bit as a boolean.  This function 
   * also works with masks that test more than one bit, but will only 
   * return true if all bits in the mask are on.
   * @param inMask A mask of the bits to check.
   * @return The value of masked bit as a boolean.
   */
  public boolean getBooleanBitValue( int inMask )
  {
    return ((data & inMask) == inMask);
  }

  /**
   * Gets the value of a specific bit as a 0 or 1.  This function 
   * also works with masks that test more than one bit, but will only 
   * return 1 if all bits in the mask are on.
   * @param inMask A mask of the bits to check.
   * @return The value of masked bit as 0 or 1.
   */  
  public int getBitValue( int inMask )
  {
    return ( (getBooleanBitValue( inMask )) ? (1) : (0) );
  }  

  /**
   * Sets the entire data value of this register.
   * @param inVal The value to which this register will be set.
   */  
  public void setValue( int inVal )
  {
    data = inVal;
    cage();
  }
  
  /**
   * Sets the value of any number of specific bits based on a mask.  All
   * other bits are left untouched.
   * @param inMask A mask of the bits to set.
   * @param inVal 0 to turn off the requested bits, any other value to turn them on.
   */
  public void setBitValue( int inMask, int inVal )
  {
    if( inVal != 0 )
      data |= inMask;
    else
      data &= ~inMask;
  }
  
  /**
   * Sets the value of any number of specific bits based on a mask.  All
   * other bits are left untouched.
   * @param inMask A mask of the bits to set.
   * @param inVal false to turn off the requested bits, true to turn them on.
   */
  public void setBitValue( int inMask, boolean inVal )
  {
    if( inVal )
      data |= inMask;
    else
      data &= ~inMask;
  }

  /**
   * Turns on specific bits based on a mask.  All other bits are left untouched.
   * @param inMask A mask of the bits to set.
   */  
  public void setBitOn( int inMask )
  {
    data |= inMask;
  }  

  /**
   * Turns off specific bits based on a mask.  All other bits are left untouched.
   * @param inMask A mask of the bits to set.
   */  
  public void setBitOff( int inMask )
  {
    data &= ~inMask;
  }

  /**
   * Increments the value of this register.
   */  
  public void increment()
  {
    data++;
    cage();
  }
  
  /**
   * Decrements the value of this register.
   */  
  public void decrement()
  {
    data--;
    cage();
  }

  /**
   * Adds a value to this register's existing value.
   * @param inVal The value to add to this register.
   */  
  public void plusEquals( int inVal )
  {
    data += inVal;
    cage();
  }  

  /**
   * Subtracts a value from this register's existing value.
   * @param inVal The value to subtract from this register.
   */  
  public void minusEquals( int inVal )
  {
    data -= inVal;
    cage();
  }

  /**
   * Performs a bitwise AND with a value and this register's existing value.
   * @param inVal The value to AND with this register.
   */  
  public void andEquals( int inVal )
  {
    data &= inVal;
    cage();
  }
  
  /**
   * Performs a bitwise OR with a value and this register's existing value.
   * @param inVal The value to OR with this register.
   */
  public void orEquals( int inVal )
  {
    data |= inVal;
    cage();
  }

  /**
   * Performs a bitwise XOR with a value and this register's existing value.
   * @param inVal The value to XOR with this register.
   */  
  public void xorEquals( int inVal )
  {
    data ^= inVal;
    cage();
  }

  /**
   * This function sets all bits that are not supposed to be in use by a register of this size to 0.
   */ 
  public void cage()
  {
    data &= 0x000000FF;
  }
}