package com.monkeycoder.monkeynes.hardware;

/**
 * This class represents a saved CPU state.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class CpuState
{
  // Registers
  private CpuRegister16 pc  = new CpuRegister16();     // Program Counter
  private CpuRegister   sp  = new CpuRegister();       // Stack Pointer
  private CpuRegister   acc = new CpuRegister();       // Accumulator
  private CpuRegister   ix  = new CpuRegister();       // X-Index
  private CpuRegister   iy  = new CpuRegister();       // Y-Index
  private CpuRegister   ps  = new CpuRegister();       // Processor Status
  
  /**
   * Creates a new CpuState where all registers are set to 0.
   */
  public CpuState()
  {
    // Do nothing...
  }

  /**
   * Creates a new CpuState where all registers are set to existing registers.
   * @param inPc The Program Counter (PC) register.
   * @param inAcc The Accumulator (ACC) register.
   * @param inIx The X-Index (IX) register.
   * @param inIy The Y-Index (IY) register.
   * @param inPs The Processor Status (PS) register.
   */  
  public CpuState( CpuRegister16 inPc, CpuRegister inSp, CpuRegister inAcc, CpuRegister inIx, CpuRegister inIy, CpuRegister inPs )
  {                
    setState( inPc, inSp, inAcc, inIx, inIy, inPs );     
  }            

  /**
   * Sets all the registers in this CpuState to existing registers.
   * @param inPc The Program Counter (PC) register.
   * @param inAcc The Accumulator (ACC) register.
   * @param inIx The X-Index (IX) register.
   * @param inIy The Y-Index (IY) register.
   * @param inPs The Processor Status (PS) register.
   */  
  public void setState( CpuRegister16 inPc, CpuRegister inSp, CpuRegister inAcc, CpuRegister inIx, CpuRegister inIy, CpuRegister inPs )
  {
    pc  = inPc;
    sp  = inSp;
    acc = inAcc;
    ix  = inIx;
    iy  = inIy;
    ps  = inPs;
  }            

  /**
   * Returns the actual Program Counter (PC) register
   * @return The Program Counter (PC) register.
   */  
  public CpuRegister16 getProgramCounter()
  {
    return pc;
  }

  /**
   * Returns the actual Stack Pointer (SP) register
   * @return The Stack Pointer (SP) register.
   */  
  public CpuRegister getStackPointer()
  {
    return sp;
  }

  /**
   * Returns the actual Accumulator (ACC) register
   * @return The Accumulator (ACC) register.
   */
  public CpuRegister getAccumulator()
  {
    return acc;
  }

  /**
   * Returns the actual X-Index (IX) register
   * @return The X-Index (IX) register.
   */
  public CpuRegister getXIndex()
  {
    return ix;
  }

  /**
   * Returns the actual Y-Index (IY) register
   * @return The Y-Index (IY) register.
   */
  public CpuRegister getYIndex()
  {
    return iy;
  }

  /**
   * Returns the actual Processor Status (PS) register
   * @return The Processor Status (PS) register.
   */
  public CpuRegister getProcessorStatus()
  {
    return ps;
  }
}                  