package com.monkeycoder.monkeynes.hardware;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.gui.*;
import com.monkeycoder.monkeynes.hardware.mapper.*;

/**
 * This represents the physical NES unit.  It contains the CPU, PPU, memory banks, the ROM cartrige, 
 * controller ports, the system clock, and a connection to the video screen.  This class ties it 
 * all together into a nice little bundle. 
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class NesHardware implements Runnable
{
  // Internal Hardware ------------------------------------
  private NesCpu               cpu           = null;
  private NesPpu               ppu           = null;
  private NesRom               rom           = null;
  private MemoryMapperFactory  mapperFactory = null;
  private NesMainMemory        mainMem       = null;
  private NesVideoMemory       vidMem        = null;
  private MemoryMapper         mapper        = null;     
  private HighResolutionTimer  clock         = null;
  
  // External Hardware ------------------------------------
  private VideoScreen          vidScreen     = null;
  private NesController[]      controllers   = new NesController[2];  // note that ports are 0 and 1!
  
  private boolean              power         = false;
  private long                 cycleCount    = 0L;       // total cycle count
  
  /**
   * Creates a new NesHardware which in turn creates all if the internal components of the NES.
   */  
  public NesHardware()
  {
    mainMem       = new NesMainMemory();
    vidMem        = new NesVideoMemory();
    cpu           = new NesCpu( mainMem, vidMem );
    ppu           = new NesPpu( vidMem );
    mapperFactory = new MemoryMapperFactory( mainMem, vidMem );
    clock         = new HighResolutionTimer( 0L );
  }

  /**
   * Sets the speed of the timer chip in the NES.
   * @param inClockSpeed The speed of the timer in Hz.
   */  
  public void setClockSpeed( long inClockSpeed )
  {
    clock.setClockSpeed( inClockSpeed );
  }

  /**
   * Connects the video screen to the hardware.
   * @param inVidScreen The video screen that will receive output from the PPU.
   */  
  public void connectVideoScreen( VideoScreen inVidScreen )
  {
    vidScreen = inVidScreen;
  }  

  /**
   * Connects a controller to the hardware through a specified controller port.
   * @param inController The controller that will be plugged into the port.
   * @param inPort The port the into which the controller will be plugged.
   */   
  public void connectController( NesController inController, int inPort )
  {
    controllers[inPort] = inController;
  }   

  /**
   * Puts a ROM cartrige into the front of the system.  Also creates a mapper
   * if the cartrige needs it.
   * <br><br>
   * Think of it as a flawless cartrige insertion every time.  No more blowing 
   * on the game, no more cleaning the contacts with a Q-Tip, no more wiggling 
   * it around.  It will just work, every time.
   * @param inRom The cartrige that is being inserted into the hardware.
   */  
  public void insertRom( NesRom inRom )
  {
    rom    = inRom;
    mapper = mapperFactory.getMapper( rom );
  }
  
  /**
   * Removes a ROM cartrige fron the system.
   * @return The ROM that was in use.
   */
  public NesRom removeRom()
  {
    NesRom tempRom = rom;
    mapper = null;
    rom    = null;
    return tempRom;
  }
  
  /**
   * Returns whether or not the power is on in the hardware.
   * @return Whether or not the power is on.
   */  
  public boolean isPowerOn()
  {
    return power;
  }

  /**
   * Called to tell the hardware that power is now on.
   */  
  public void powerOn()
  {
    cpu.powerOn();
    ppu.powerOn();
    if( mapper != null )
      mapper.powerOn(); 
      
    power = true;   
  }

  /**
   * Called to tell the hardware that power is now off.
   */    
  public void powerOff()
  {
    cpu.powerOff();
    ppu.powerOff();
    if( mapper != null )
      mapper.powerOff();
      
    power = false;  
  }

  /**
   * Called to tell the hardware to reset.
   */  
  public void reset()
  {
    
  }
  
  // ----------------------------------------------------------------------------------
  // The main timer loop --------------------------------------------------------------
  // ----------------------------------------------------------------------------------
  /**
   * Threaded run method so the hardware can operate in it's own thread.
   * This allows hardware to be created and placed into a seperate thread
   * and started up and allowed to run on its own.  This method manages 
   * the operation of the clock and tells all attached components to do
   * cycles as appropriate.
   */  
  public void run()
  {
    long startTime  = System.currentTimeMillis();
    long stopTime   = 0L;
    long i          = 0L;
    long cycleBatch = 0L;
    
    cycleCount      = 0L;
    clock.startClock();
    
    while( power )
    {
      cycleBatch = clock.getClocksPast();
      
      // Do all the cycles in the batch one at a time
      if( cycleBatch != 0 )
      {
        for( i = 0; i < cycleBatch; i++ )
        {
          if( (i % 2) == 1 )
          {
            cpu.doCycle();
            mapper.doCycle();
          }
            
          ppu.doCycle();
        }

        // Catch a little break
        sleepPause( 1 );

        DebugLog.log( 5, "Cycles Complete: " + cycleBatch );
        cycleCount += cycleBatch;
      }
      else
      {
        // I can do other upkeep things here.
        
        // Catch a little break
        sleepPause( 1 );
      }  
    }    
    
    clock.stopClock();
    stopTime = System.currentTimeMillis();
    DebugLog.log( 3, "Total Cycles Complete: " + cycleCount );  
    DebugLog.log( 3, "Total Time:            " + (stopTime - startTime) + "ms" );  
    DebugLog.log( 3, "Effective Clock Speed: " + (long)((double)cycleCount / (double)(stopTime - startTime) * 1000.0) + "hz" );
  }  
  // ----------------------------------------------------------------------------------
  // END The main timer loop ----------------------------------------------------------
  // ----------------------------------------------------------------------------------
  
  /**
   * Convenience method for putting a thread to sleep for a 
   * specified number of milliseconds.  This method does its
   * own exception trapping allowing this function to be called
   * anywhere without the use of a seperate <code>try...catch</code>
   * block.
   * @param inMs The number of milliseconds to sleep.
   */
  private void sleepPause( int inMs )
  {
    try 
    { 
      Thread.sleep( inMs ); 
    } 
    catch( InterruptedException ie ) 
    {
      // Do nothing
    }
  }
  
  
  // Helper Access Functions ------------------------------------
  /** 
   * Returns the CPU created by and in use by this hardware.
   * @return The CPU in use by this hardware.
   */
  public NesCpu getCpu()
  {
    return cpu;
  }
  
  /** 
   * Returns the PPU created by and in use by this hardware.
   * @return The PPU in use by this hardware.
   */
  public NesPpu getPpu()
  {
    return ppu;
  }

  /** 
   * Returns the VideoScreen that is attached to this hardware.
   * @return The VideoScreen attached to this hardware.
   */  
  public VideoScreen getVideoScreen()
  {
    return vidScreen;
  }
  
  /** 
   * Returns the NesRom currently in the ROM slot of this hardware.
   * @return The NesRom currently in the ROM slot of this hardware.
   */
  public NesRom getRom()
  {
    return rom;
  }

  /** 
   * Returns the MemoryMapper in use by the NesRom.
   * @return The MemoryMapper in use by the NesRom.
   */  
  public MemoryMapper getMemoryMapper()
  {
    return mapper;
  }

  /** 
   * Returns the Main Memory bank created by and in use by this hardware.
   * @return The Main Memory bank in use by this hardware.
   */  
  public NesMainMemory getMainMemory()
  {
    return mainMem;
  }

  /** 
   * Returns the Video Memory bank created by and in use by this hardware.
   * @return The Video Memory bank in use by this hardware.
   */  
  public NesVideoMemory getVideoMemory()
  {
    return vidMem;
  }
  
  // ------------------------------------------------------------
  
  /**
   * Creates a huge text readout detailing everything currently connected 
   * to the hardware.
   * @return A String containing details on everything connected to the hardware.
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    
    sb.append( "NesHardware -------------------------\n" );
    
    // Clock Speed
    sb.append( "  Clock Speed: " );
    sb.append( clock.getClockSpeed() );
    sb.append( "hz\n" );
    
    // Video Screen
    sb.append( "  VideoScreen: " );
    if( vidScreen != null )
      sb.append( "connected\n" );
    else
      sb.append( "not connected\n" );
    
    // Controllers
    sb.append( "  Controllers\n" );
    for( int i = 0; i < controllers.length; i++ )
    {
      sb.append( "    Port " );
      sb.append( i );
      sb.append( ": " );
      if( controllers[i] != null )
      {
        sb.append( controllers[i].getTypeName() );
        sb.append( " [" );
        sb.append( controllers[i].getType() );        
        sb.append( "]" );
      }  
      else
        sb.append( "not connected" );
      sb.append( "\n" );  
    }  
    
    // ROM
    sb.append( "  ROM\n" );
    if( rom != null )
    {
      sb.append( rom.toString() );
      sb.append( "\n" );
    }  
    else
      sb.append( "  not inserted\n" );
      
    // Mapper
    sb.append( "  Mapper: " );
    if( mapper != null )
      sb.append( mapper.toString() );
    else
      sb.append( "  information not available" );
      
    
    return sb.toString();
  }
  
}